!{\src2tex{textfont=tt}}
!!****f* ABINIT/jbessel
!! NAME
!! jbessel
!!
!! FUNCTION
!! Compute spherical Bessel function j_l(x) or derivative
!!
!! COPYRIGHT
!! Copyright (C) 1998-2005 ABINIT group (MT)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!! INPUTS
!!  ll=l-order of the Bessel function
!!  optder=1 if derivative is requested
!!  xx=where to compute j_l
!!
!! OUTPUT
!!  bes= Bessel function j_l at xx
!!  besp= deivative of j_l at xx (only if optder=1)
!!
!! PARENTS
!!      opt_proj
!!
!! SOURCE

 subroutine jbessel(bes,besp,ll,optder,xx)

 use defs_basis

 implicit none


!Arguments ---------------------------------------------
 integer :: ll,optder
 real(dp) :: bes,besp,xx

!Local variables ---------------------------------------
 integer,parameter :: imax=40
 real(dp),parameter :: prec=1.d-15
 integer :: ii,il
 real(dp) :: fact,factp,jn,jnp,jr,xx2,xxinv

! *********************************************************************

 if (xx<1.d0) then
  xx2=0.5d0*xx*xx
  fact=one
  do il=1,ll
   fact=fact*xx/dble(2*il+1)
  end do
  if (optder==1) factp=fact*xx/dble(2*ll+3)
  jn=one;jr=one
  do ii=1,imax
   jr=-jr*xx2/dble(ii*(2*(ll+ii)+1))
   if (abs(jr)<prec) goto 10
   jn=jn+jr
  end do
  stop 'Error: Bessel function did not converge !'
10 bes=jn*fact
  if (optder==1) then
   jnp=one;jr=one
   do ii=1,imax
    jr=-jr*xx2/dble(ii*(2*(ll+ii)+3))
    if (abs(jr)<prec) goto 11
    jnp=jnp+jr
   end do
   stop 'Error: Bessel function did not converge !'
11 besp=-jnp*factp+bes*ll/xx
  end if
 else
  xxinv=1.d0/xx
  jn = sin(xx)*xxinv
  jnp=(-cos(xx)+jn)*xxinv
  do il=2,ll+1
   jr=-jn+dble(2*il-1)*jnp*xxinv
   jn=jnp;jnp=jr
  end do
  bes=jn
  if (optder==1) besp=-jnp+jn*ll*xxinv
 end if

 end subroutine jbessel

!!***
